@@ -12,14 +12,14 @@
 #include "flags_fs.h"
 
 void
-print_kdev_t(kdev)
+mprint_kdev_t(kdev)
 	kdev_t kdev;
 {
 	mprintf("%3d,%-3d", MAJOR(kdev), MINOR(kdev));
 }
 
 void
-print_dev_t(dev)
+mprint_dev_t(dev)
 	dev_t dev;
 {
 	mprintf("%3d,%-3d", MAJOR(dev), MINOR(dev));
@@ -43,9 +43,9 @@ print_buffer_head(addr, full, lru)
 	memread(addr, sizeof(struct buffer_head), &buf, "buffer_head");
 	mprintf(FPTR " ", addr);
 	mprintf("%8lx %4x %3x ", buf.b_blocknr, buf.b_size, ATOMIC_READ(buf.b_count));
-	print_kdev_t(buf.b_dev);
+	mprint_kdev_t(buf.b_dev);
 	mprintf(" ");
-	print_kdev_t(buf.b_rdev);
+	mprint_kdev_t(buf.b_rdev);
 	mprintf(" " FPTR " " FPTR, buf.b_data, buf.b_page);
 	mprintbit(bh_state_bits_bitname, buf.b_state);
 	mprintf("\n");
@@ -106,14 +106,14 @@ print_dentry(addr, sw, subdir, parent)
 	mprintf("%lx ", de.d_vfs_flags);
 #endif
 	if (de.d_name.name == ((struct dentry *)addr)->d_iname)
-		mprint_str(de.d_iname, sizeof(de.d_iname));
+		mprintstr(de.d_iname, sizeof(de.d_iname));
 	else if (de.d_name.name == NULL)
 		mprintf("[null]");
 	else {
 		char qbuf[256];
 		int len = de.d_name.len < sizeof(qbuf)? de.d_name.len: sizeof(qbuf);
 		memread((addr_t)de.d_name.name, len, qbuf, "dentry.d_name");
-		mprint_str(qbuf, len);
+		mprintstr(qbuf, len);
 	}
 	mprintf("\n");
 
@@ -133,7 +133,7 @@ print_dentry(addr, sw, subdir, parent)
 void
 prhead_file()
 {
-	mprintf(SPTR" "SPTR" "SPTR" MOD      POS CNT   UID   GID ERR "SPTR" FLAGS\n",
+	mprintf(SPTR" "SPTR" "SPTR" MOD       POS CNT   UID   GID ERR "SPTR" FLAGS\n",
 		"ADDR", "DENTRY", "F_OP", "PRIVATE");
 }
 
@@ -156,7 +156,7 @@ print_file(addr, full)
 	memread(addr, sizeof(f), &f, "file");
 	mprintf(FPTR " ", addr);
 
-	mprintf(FPTR " " FPTR "  ", f.f_dentry, f.f_op);
+	mprintf(FPTR " " FPTR " ", f.f_dentry, f.f_op);
 	switch (f.f_mode) {
 	case 0:	mprintf(" -");	break;
 	case 1: mprintf("r ");	break;
@@ -164,10 +164,19 @@ print_file(addr, full)
 	case 3: mprintf("rw");	break;
 	default:mprintf(" ?");	break;
 	}
-	mprintf(" %8Lx %3x %5d %5d %3d", f.f_pos, ATOMIC_READ(f.f_count), f.f_uid, f.f_gid, f.f_error);
+	mprintf(" %10Lx %3x %5d %5d %3d", f.f_pos, ATOMIC_READ(f.f_count), f.f_uid, f.f_gid, f.f_error);
 	mprintf(" " FPTR, f.private_data);
-	mprintbit(fflags, f.f_flags);
+	mprintbit(fflags, f.f_flags & ~3);
 	mprintf("\n");
+
+	if (full) {
+		mprintf("\tvfsmnt:     " FPTR "\n", f.f_vfsmnt);
+		mprintf("\treada:      %lx  %lx  %lx  %lx  %lx\n",
+			f.f_reada, f.f_ramax, f.f_raend,
+			f.f_ralen, f.f_rawin);
+		mprintf("\towener.pid: %x\n", f.f_owner.pid);
+		mprintf("\tversion:    %lx\n", f.f_version);
+	}
 	return (addr_t)f.f_list.next;
 }
 
@@ -175,7 +184,7 @@ void
 prhead_filesystem()
 {
 	mprintf(SPTR"        NAME "SPTR" "SPTR" FLAG\n",
-		"ADDR", "READ_S", "OWNER");
+		"ADDR", "READ_SP", "OWNER");
 }
 
 addr_t
@@ -202,7 +211,7 @@ print_file_system_type(addr)
 void
 prhead_inode()
 {
-	mprintf(SPTR"      INO CNT   DEV  LINK MODE   UID       SIZE "SPTR" HLDD\n",
+	mprintf(SPTR"        INO CNT   DEV  LINK MODE   UID       SIZE "SPTR" HLDD\n",
 		"ADDR", "MAPPING");
 }
 
@@ -212,13 +221,14 @@ print_inode(addr, full)
 	int full;
 {
 	struct inode inode;
+	const char *p;
 
 	memread(addr, sizeof(struct inode), &inode, "inode");
 	mprintf(FPTR " ", addr);
-	mprintf("%8ld %3x ", inode.i_ino, ATOMIC_READ(inode.i_count));
-	print_kdev_t(inode.i_dev);
+	mprintf("%10ld %3x ", inode.i_ino, ATOMIC_READ(inode.i_count));
+	mprint_kdev_t(inode.i_dev);
 	mprintf(" %3x ", inode.i_nlink);
-	pmode(inode.i_mode);
+	mprint_mode(inode.i_mode);
 	mprintf(" %5d %10Lx", inode.i_uid, inode.i_size);
 	mprintf(" " FPTR, inode.i_mapping);
 	mprintf(" %c%c%c%c\n",
@@ -228,19 +238,46 @@ print_inode(addr, full)
 		(inode.i_dirty_buffers.next!=&((struct inode *)addr)->i_dirty_buffers)? 'D': '-');
 
 	if (full) {
-		mprintf("\tstate:%lx  flags:%x\n", inode.i_state, inode.i_flags);
-		mprintf("\trdev:");
-		print_kdev_t(inode.i_rdev);
-		mprintf("  blksize:%ld  blocks:%ld  version:%ld\n", inode.i_blksize, inode.i_blocks, inode.i_version);
-		mprintf("\tatime:%08lx  mtime:%08lx  ctime:%08lx\n", inode.i_atime, inode.i_mtime, inode.i_ctime);
-		mprintf("\tSEM (count:%d  sleepers:%d)\n", ATOMIC_READ(inode.i_sem.count), inode.i_sem.sleepers);
-		mprintf("\top:" FPTR "  fop:" FPTR "  sb:" FPTR "  flock:" FPTR "  data:" FPTR "\n", inode.i_op, inode.i_fop, inode.i_sb, inode.i_flock, (addr_t)&((struct inode *)addr)->i_data);
-		mprintf("\tbdev:%lx  gen:%u  u:" FPTR "\n", inode.i_bdev, inode.i_generation, (addr_t)&((struct inode *)addr)->u);
-		mprintf("\tlist: " FPTR " " FPTR "\n",
-			inode.i_list.next == 0? 0:
-			(addr_t)inode.i_list.next - OFFSET(struct inode,i_list),
-			inode.i_list.prev == 0? 0:
-			(addr_t)inode.i_list.prev - OFFSET(struct inode,i_list));
+		mprintf("\trdev:      ");
+		mprint_kdev_t(inode.i_rdev);
+		mprintf("\n");
+		mprintf("\t[amc]time: %08lx  %08lx  %08lx  (date)\n",
+			inode.i_atime,
+			inode.i_mtime,
+			inode.i_ctime);
+		mprintf("\tblocks:    %ld  (* blksize %ld)\n", inode.i_blocks, inode.i_blksize);
+		mprintf("\tversion:   %ld\n", inode.i_version);
+		mprintf("\tsem:       ");
+		mprint_semaphore(&inode.i_sem);
+		mprintf("\n");
+		mprintf("\top:        " FPTR "  (operation -i)  ", inode.i_op);
+		p = getsymstr((addr_t)inode.i_op);
+		if (p)
+			mprintf("(%s)\n", p);
+		else
+			mprintf("\n");
+		mprintf("\tfop:       " FPTR "  (operation -f)  ", inode.i_fop);
+		p = getsymstr((addr_t)inode.i_fop);
+		if (p)
+			mprintf("(%s)\n", p);
+		else
+			mprintf("\n");
+		mprintf("\tsb:        " FPTR "  (super_block)\n", inode.i_sb);
+		mprintf("\tflock:     " FPTR "  (file_lock)\n", inode.i_flock);
+		if (inode.i_pipe)
+		mprintf("\tpipe:      " FPTR "\n", inode.i_pipe);
+		if (inode.i_bdev)
+		mprintf("\tbdev:      " FPTR "  (block_device)\n", inode.i_bdev);
+#ifdef _inode_has_i_cdev
+		if (inode.i_cdev)
+		mprintf("\tcdev:      " FPTR "  (char_device)\n", inode.i_cdev);
+#endif
+		mprintf("\tstate:     %lx ", inode.i_state);
+		mprintbit(istate, inode.i_state);
+		mprintf("\n\tflags:     %x\n", inode.i_flags);
+		mprintf("\tgen:       %u\n", inode.i_generation);
+		mprintf("\tu:        *" FPTR "  = " FPTR "\n",
+			addr + OFFSET(struct inode, u), inode.u.generic_ip);
 		mprintf("\n");
 	}
 	return (addr_t)inode.i_hash.next;
@@ -249,7 +286,7 @@ print_inode(addr, full)
 void
 prhead_superblock()
 {
-	mprintf(SPTR"   DEV   BSIZE        TYPE "SPTR"    MAGIC "SPTR" "SPTR" FLAGS\n",
+	mprintf(SPTR"   DEV   BSIZE D        TYPE "SPTR" "SPTR" "SPTR" LDIF FLAGS\n",
 		"ADDR", "S_OP", "ROOT", "BDEV");
 }
 
@@ -265,61 +302,92 @@ print_superblock(addr, full)
 	memread(addr, sizeof(struct super_block), &sb, "super_block");
 	mprintf(FPTR " ", addr);
 
-	print_kdev_t(sb.s_dev);
-	mprintf(" %5lx", sb.s_blocksize);
+	mprint_kdev_t(sb.s_dev);
+	mprintf(" %5lx %x ", sb.s_blocksize, sb.s_dirt);
 
 	if (sb.s_type) {
 		memread((addr_t)sb.s_type, sizeof(fstype), &fstype, "file system type");
 		memread((addr_t)fstype.name, sizeof(buf), buf, "file system type name");
-		mprintf(" %11s", buf);
+		mprintf("%11s", buf);
 	} else {
-		mprintf(" %11s", "-");
+		mprintf("%11s", "-");
 	}
 
-	mprintf(" " FPTR, sb.s_op);
-	mprintf(" %8lx", sb.s_magic);
-	mprintf(" " FPTR " " FPTR, sb.s_root, sb.s_bdev);
-#if 0
-	if (sb.s_dirty.next == &((struct super_block *)addr)->s_dirty) {
-		mprintf(SPTR " ", "-");
-	} else {
-		mprintf(" " FPTR, sb.s_dirty.next);
-	}
-	if (sb.s_dirty.prev == &((struct super_block *)addr)->s_dirty) {
-		mprintf(SPTR " ", "-");
-	} else {
-		mprintf(" " FPTR, sb.s_dirty.prev);
-	}
-#endif
+	mprintf(" " FPTR " " FPTR " " FPTR, sb.s_op, sb.s_root, sb.s_bdev);
+
+#define ISEMPTY(x)	((addr_t)sb.x.next == addr+OFFSET(struct super_block,x))
+	mprintf(" %c%c%c%c",
+		ISEMPTY(s_list)         ? '-': 'l',
+		ISEMPTY(s_dirty)        ? '-': 'd',
+		ISEMPTY(s_locked_inodes)? '-': 'i',
+		ISEMPTY(s_files)        ? '-': 'f');
 
 	mprintbit(sflags, sb.s_flags);
 	mprintf("\n");
 
 	if (full) {
-		mprintf("\ts_blocksize_bits: %x\n", sb.s_blocksize_bits);
-		mprintf("\tdirt: %x\n", sb.s_dirt);
+		mprintf("\tmaxbytes:         %llx\n", sb.s_maxbytes);
 		mprintf("\tdquot_operations: " FPTR "\n", sb.dq_op);
-		mprintf("\tlock:");
-		__print_semaphore(&sb.s_lock);
+		mprintf("\tmagic:            %lx\n", sb.s_magic);
+		mprintf("\tlock:             ");
+		mprint_semaphore(&sb.s_lock);
+		mprintf("\n");
+#ifdef _super_block_has_s_count
+		mprintf("\tcount:            %x\n", sb.s_count);
+#endif
+#ifdef _super_block_has_s_active
+		mprintf("\tactive:           %x\n", ATOMIC_READ(sb.s_active));
+#endif
+#define SEMPTY(list) \
+	(addr + OFFSET(struct super_block, list) == (addr_t)sb.list.next)
+		mprintf("\tdirty:            ");
+		if (SEMPTY(s_dirty))
+			mprintf(SPTR " " SPTR "\n", "-", "-");
+		else
+			mprintf(FPTR " " FPTR "\n",
+				sb.s_dirty.next, sb.s_dirty.prev);
+		mprintf("\tlocked_inodes:    ");
+		if (SEMPTY(s_locked_inodes))
+			mprintf(SPTR " " SPTR "\n", "-", "-");
+		else
+			mprintf(FPTR " " FPTR "\n",
+				sb.s_locked_inodes.next,
+				sb.s_locked_inodes.prev);
+		mprintf("\tfiles:            ");
+		if (SEMPTY(s_files))
+			mprintf(SPTR " " SPTR "\n", "-", "-");
+		else
+			mprintf(FPTR " " FPTR "\n",
+				sb.s_files.next, sb.s_files.prev);
+		mprintf("\tu:                " FPTR "\n",
+			addr + OFFSET(struct super_block, u));
+	}
+
+	if (full >= 2) {
+		addr_t faddr = (addr_t)sb.s_files.next;
+		prhead_file();
+		while (faddr != addr + OFFSET(struct super_block, s_files)) {
+			faddr = print_file(faddr - OFFSET(struct file, f_list), 0);
+		}
 		mprintf("\n");
-		mprintf("\tu: " FPTR "\n", addr + OFFSET(struct super_block, u));
 	}
 
 	return (addr_t)sb.s_list.next;
 }
 
 addr_t
-print_superblock_union(addr, fstype, func)
+print_superblock_union(addr, fstype, func, arg)
 	addr_t addr;
 	addr_t fstype;
 	addr_t (*func)();
+	int arg;
 {
 	struct super_block sb;
 
 	memread(addr, sizeof(struct super_block), &sb, "super_block");
 
 	if ((addr_t)sb.s_type == fstype)
-		func(sb.u, addr + OFFSET(struct super_block, u));
+		func(sb.u.generic_sbp, addr + OFFSET(struct super_block, u), arg);
 	return (addr_t)sb.s_list.next;
 }
 
@@ -377,9 +445,9 @@ print_files_stat(addr)
 	struct files_stat_struct filst;
 
 	memread(addr, sizeof(filst), &filst, "files_stat");
-	mprintf("nr_files:      %5x\n", filst.nr_files);
-	mprintf("nr_free_files: %5x\n", filst.nr_free_files);
-	mprintf("max_files:     %5x\n", filst.max_files);
+	mprintf("nr_files:      %5x  (%d)\n", filst.nr_files, filst.nr_files);
+	mprintf("nr_free_files: %5x  (%d)\n", filst.nr_free_files, filst.nr_free_files);
+	mprintf("max_files:     %5x  (%d)\n", filst.max_files, filst.max_files);
 }
 
 void
@@ -396,12 +464,27 @@ print_address_space(addr, plist)
 	mprintf("ADDR:         " FPTR "\n", addr);
 	memread(addr, sizeof(as), &as, "address_space");
 
-	mprintf("clean_pages:  " FPTR "  " FPTR "\n",
-		as.clean_pages.next, as.clean_pages.prev);
-	mprintf("dirty_pages:  " FPTR "  " FPTR "\n",
-		as.dirty_pages.next, as.dirty_pages.prev);
-	mprintf("locked_pages: " FPTR "  " FPTR "\n",
-		as.locked_pages.next, as.locked_pages.prev);
+#undef ISEMPTY
+#define ISEMPTY(list) \
+	((addr_t)as.list.next == addr + OFFSET(struct address_space, list))
+	mprintf("clean_pages:  ");
+	if (ISEMPTY(clean_pages))
+		mprintf(SPTR "  " SPTR "\n", "-", "-");
+	else
+		mprintf(FPTR "  " FPTR "\n",
+			as.clean_pages.next, as.clean_pages.prev);
+	mprintf("dirty_pages:  ");
+	if (ISEMPTY(dirty_pages))
+		mprintf(SPTR "  " SPTR "\n", "-", "-");
+	else
+		mprintf(FPTR "  " FPTR "\n",
+			as.dirty_pages.next, as.dirty_pages.prev);
+	mprintf("locked_pages: ");
+	if (ISEMPTY(locked_pages))
+		mprintf(SPTR "  " SPTR "\n", "-", "-");
+	else
+		mprintf(FPTR "  " FPTR "\n",
+			as.locked_pages.next, as.locked_pages.prev);
 
 	mprintf("nrpages:      %8lx\n", as.nrpages);
 	mprintf("ops:          " FPTR "\n", as.a_ops);
@@ -454,7 +537,7 @@ print_char_device(addr)
 	mprintf(FPTR " ", addr);
 
 	mprintf("%5x ", ATOMIC_READ(cd.count));
-	print_dev_t(cd.dev);
+	mprint_dev_t(cd.dev);
 	mprintf(" %4x ", ATOMIC_READ(cd.openers));
 	mprintf("%5d %3d\n", ATOMIC_READ(cd.sem.count), cd.sem.sleepers);
 	return (addr_t)cd.hash.next;
@@ -493,7 +576,7 @@ print_block_device(addr, eflag)
 
 	mprintf(FPTR " ", addr);
 	mprintf("%5x ", ATOMIC_READ(bd.bd_count));
-	print_dev_t(bd.bd_dev);
+	mprint_dev_t(bd.bd_dev);
 	mprintf(" %4x " FPTR, bd.bd_openers, bd.bd_op);
 	mprintf(" %5d %3d", ATOMIC_READ(bd.bd_sem.count), bd.bd_sem.sleepers);
 #if SUBLEVEL>=15
@@ -551,13 +634,32 @@ print_file_lock(addr, full, chain)
 	mprintf("\n");
 
 	if (full) {
-		mprintf("  notify: " FPTR "\n", fl.fl_notify);
-		mprintf("  insert: " FPTR "\n", fl.fl_insert);
-		mprintf("  rename: " FPTR "\n", fl.fl_remove);
-		mprintf("  fasync: " FPTR "\n", fl.fl_fasync);
+		mprintf("  next:       " FPTR "\n", fl.fl_next);
+		mprintf("  block:      ");
+		if ((addr_t)fl.fl_block.next == addr + OFFSET(struct file_lock, fl_block)) {
+			mprintf(SPTR " " SPTR "\n", "-", "-");
+		} else {
+			mprintf(FPTR " " FPTR "\n",
+				fl.fl_block.next, fl.fl_block.prev);
+		}
+		mprintf("  wait:       ");
+		if ((addr_t)fl.fl_wait.task_list.next == addr + OFFSET(struct file_lock, fl_wait.task_list)) {
+			mprintf(SPTR " " SPTR "\n", "-", "-");
+		} else {
+			mprintf(FPTR " " FPTR "\n",
+				fl.fl_wait.task_list.next,
+				fl.fl_wait.task_list.prev);
+		}
+		mprintf("  not/ins/rem:" FPTR " " FPTR " " FPTR "\n",
+			fl.fl_notify, fl.fl_insert, fl.fl_remove);
+		mprintf("  fasync:     " FPTR "\n", fl.fl_fasync);
 #ifdef _file_lock_has_fl_break_time
 		mprintf("  break_time: %8lx\n", fl.fl_break_time);
 #endif
+		mprintf("  nfs_fl:     %x  %x  %lx\n",
+			fl.fl_u.nfs_fl.state,
+			fl.fl_u.nfs_fl.flags,
+			fl.fl_u.nfs_fl.host);
 	}
 
 	switch (chain) {
