@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 ASANO Masahiro
+ * Copyright (C) 2000-2003 ASANO Masahiro
  */
 
 #include "def.h"
@@ -11,7 +11,8 @@
 void
 prhead_neighbour()
 {
-	mprintf(SPTR" "SPTR" "SPTR" "SPTR" "SPTR"\n", "ADDR", "TBL", "PARMS", "DEV", "HH");
+	mprintf(SPTR" "SPTR" "SPTR" "SPTR" "SPTR" "SPTR" "SPTR"\n", "ADDR",
+		"TBL", "PARMS", "DEV", "HH", "OUTPUT", "OPS");
 }
 
 addr_t
