@@ -5,9 +5,11 @@
 #include <unistd.h>
 #include "crash.h"
 
+#include "flags_fs.h"
+
 PRIVATE addr_t operations();
 const commandtable_t command_operations =
-	{"operations", operations, "-a|-b|-d|-f|-i|-s|-q|-v address", "print address_space, block_device, dentry, file, inode, super, dquot or vm operations"};
+	{"operations", operations, "{-a|-b|-d|-f|-n|-i|-s|-q|-v} address", "print address_space, block_device, dentry, file, inode, nfs_rpc, super, dquot or vm operations"};
 
 PRIVATE const char *address_space_op[] = {
 	"ADDRESS SPACE OPERATIONS",
@@ -15,6 +17,9 @@ PRIVATE const char *address_space_op[] = {
 	"bmap",
 #if SUBLEVEL>=15
 	"flushpage", "releaspage", "direct_IO",
+#endif
+#ifdef _address_space_operations_has_removepage
+	"removepage",
 #endif
 	NULL
 };
@@ -22,6 +27,15 @@ PRIVATE const char *address_space_op[] = {
 PRIVATE const char *block_device_op[] = {
 	"BLOCK DEVICE OPERATIONS",
 	"open", "release", "ioctl", "check_media_change", "revalidate",
+#ifdef _block_device_operations_has_owner
+	"owner",
+#endif
+	NULL
+};
+
+PRIVATE const char *dentry_op[] = {
+	"DENTRY OPERATIONS",
+	"revalidate", "hash", "compare", "delete", "release", "iput",
 	NULL
 };
 
@@ -42,16 +56,50 @@ PRIVATE const char *inode_op[] = {
 	"create", "lookup", "link", "unlink", "symlink",
 	"mkdir", "rmdir", "mknod", "rename", "readlink", "follow_link",
 	"truncate", "permission", "revalidate", "setattr", "getattr",
+#ifdef _inode_operations_has_setxattr
+	"setxattr",
+#endif
+#ifdef _inode_operations_has_getxattr
+	"getxattr",
+#endif
+#ifdef _inode_operations_has_listxattr
+	"listxattr",
+#endif
+#ifdef _inode_operations_has_removexattr
+	"removexattr",
+#endif
+	NULL
+};
+
+PRIVATE const char *nfs_rpc_op[] = {
+	"NFS_RPC OPERATIONS",
+	"version", "getroot", "getattr", "setattr", "lookup", "access",
+	"readlink", "read", "write", "commit", "create", "remove",
+	"unlink_setup", "unlink_done", "rename", "link", "symlink",
+	"mkdir", "rmdir", "readdir", "mknod", "statfs", "decode_dirent",
 	NULL
 };
 
 PRIVATE const char *super_op[] = {
 	"SUPER OPERATIONS",
+#ifdef _super_operations_has_alloc_inode
+	"alloc_inode",
+#endif
+#ifdef _super_operations_has_destroy_inode
+	"destroy_inode",
+#endif
 	"read_inode", "read_inode2", "dirty_inode", "write_inode", "put_inode",
-	"delete_inode", "put_super", "write_super", "write_super_lockfs",
-	"unlockfs", "statfs", "remount_fs", "clear_inode", "umount_begin",
+	"delete_inode", "put_super", "write_super",
+#ifdef _super_operations_has_sync_fs
+	"sync_fs",
+#endif
+	"write_super_lockfs", "unlockfs", "statfs", "remount_fs",
+	"clear_inode", "umount_begin",
 #if SUBLEVEL>=15
 	"fh_to_dentry", "dentry_to_fh",
+#endif
+#ifdef _super_operations_has_show_options
+	"show_options",
 #endif
 	NULL
 };
@@ -63,12 +111,6 @@ PRIVATE const char *dquot_op[] = {
 	NULL
 };
 
-PRIVATE const char *dentry_op[] = {
-	"DENTRY OPERATIONS",
-	"revalidate", "hash", "compare", "delete", "release", "iput",
-	NULL
-};
-
 PRIVATE const char *vm_op[] = {
 	"VM OPERATIONS STRUCT",
 	"open", "close", "nopage",
@@ -101,7 +143,7 @@ operations()
 	const char **op = NULL;
 	addr_t addr;
 
-	while ((c = getopt(argcnt, args, "abdfiqsv")) != EOF) {
+	while ((c = getopt(argcnt, args, "abdfinqsv")) != EOF) {
 		switch (c) {
 		case 'a':
 			op = address_space_op;
@@ -115,6 +157,9 @@ operations()
 		case 'i':
 			op = inode_op;
 			break;
+		case 'n':
+			op = nfs_rpc_op;
+			break;
 		case 's':
 			op = super_op;
 			break;
