@@ -7,7 +7,7 @@
 
 PRIVATE addr_t page(), vm_area_struct(), zone(), pglist_data();
 const commandtable_t command_page =
-	{"page", page, "[-a] address | -n #", "print page (or mem_map_t) table\n  -n  specify page number"};
+	{"page", page, "[-a] [address] | -n PFN", "print page (or mem_map_t) table\n  -n  specify page frame number"};
 const commandtable_t command_vm_area_struct =
 	{"vm_area_struct", vm_area_struct, "address", "print vm_area_struct table"};
 const commandtable_t command_zone =
@@ -20,7 +20,7 @@ extern addr_t print_page();
 extern addr_t print_vm_area_struct();
 extern addr_t print_zone();
 extern addr_t print_pglist_data();
-extern int sizeof_page();
+extern int sizeof_page;
 addr_t page_hash_table_addr;
 addr_t page_hash_bits_addr;
 addr_t mem_map_addr, mem_map;
@@ -29,7 +29,7 @@ addr_t pgdat_list_addr;
 PRIVATE addr_t
 page()
 {
-	addr_t addr, start, page_hash_table;
+	addr_t addr = 0, start, page_hash_table;
 	int i, c;
 	unsigned int page_hash_bits;
 	int page_hash_size;
@@ -54,12 +54,11 @@ page()
 	}
 
 	if (nflag) {
-		int size = sizeof_page();
 		if (argcnt == optind)
 			THROW(usage);
 		prhead_page();
 		for (i = optind; i < argcnt; i++) {
-			addr = print_page(mem_map + size * getvalue(args[i]), 0);
+			addr = print_page(mem_map + sizeof_page * getvalue(args[i]), 0);
 		}
 		return addr;
 	}
@@ -120,12 +119,15 @@ vm_area_struct()
 PRIVATE addr_t
 zone()
 {
-	int c;
+	int c, full = 0;
 	addr_t addr = 0;
 	extern void prhead_zone();
 
-	while ((c = getopt(argcnt, args, "")) != EOF) {
+	while ((c = getopt(argcnt, args, "f")) != EOF) {
 		switch (c) {
+		case 'f':
+			full = 1;
+			break;
 		default:
 			THROW(usage);
 		}
@@ -140,7 +142,7 @@ zone()
 		memread(pgdat_list_addr, sizeof(zaddr), &zaddr, "pgdat_list");
 		prhead_zone();
 		for (i = 0; (addr = get_zone_from_pglist_data(zaddr, i)); i++) {
-			print_zone(addr, 0);
+			print_zone(addr, full);
 		}
 		return addr;
 	}
