@@ -1,4 +1,5 @@
 #include "crash.h"
+#include "flags_sched.h"
 
 extern const commandtable_t
 	command_address_space,
@@ -17,21 +18,37 @@ extern const commandtable_t
 	command_kmem_cache,
 	command_lv,
 	command_neighbour,
+	command_nfs_inode_info,
+	command_nfs_page,
+	command_nfs_reqlist,
+	command_nfs_server,
+	command_nlm_block,
+	command_nlm_file,
+	command_nlm_rqst,
 	command_operations,
 	command_page,
 	command_pglist_data,
 	command_pv,
 	command_request_queue,
 	command_rtable,
+#ifdef _sighand_struct_has_action
+	command_sighand_struct,
+#else
+	command_signal_struct,
+#endif
 	command_slab,
 	command_sock,
 	command_socket,
+	command_svc_client,
+	command_svc_export,
 	command_swap_info,
 	command_tq_struct,
 	command_timer_vec,
+	command_tty_struct,
 	command_usb_bus,
 	command_usb_device,
 	command_usb_driver,
+	command_user_struct,
 	command_vg,
 	command_vm_area_struct,
 	command_vm_struct,
@@ -54,30 +71,52 @@ const commandtable_t *commandtable_vers[] = {
 	&command_files_stat,
 	&command_inode,
 	&command_kmem_cache,
-#ifdef CONFIG_BLK_DEV_LVM
+#if defined(CONFIG_BLK_DEV_LVM) || defined(CONFIG_BLK_DEV_LVM_MODULE)
 	&command_lv,
 #endif
 	&command_neighbour,
+#if defined(CONFIG_NFS_FS) || defined(CONFIG_NFS_FS_MODULE)
+	&command_nfs_inode_info,
+	&command_nfs_page,
+	&command_nfs_reqlist,
+	&command_nfs_server,
+#endif
+#if defined(CONFIG_LOCKD) || defined(CONFIG_LOCKD_MODULE)
+	&command_nlm_block,
+	&command_nlm_file,
+	&command_nlm_rqst,
+#endif
 	&command_operations,
 	&command_page,
 	&command_pglist_data,
-#ifdef CONFIG_BLK_DEV_LVM
+#if defined(CONFIG_BLK_DEV_LVM) || defined(CONFIG_BLK_DEV_LVM_MODULE)
 	&command_pv,
 #endif
 	&command_request_queue,
 	&command_rtable,
+#ifdef _sighand_struct_has_action
+	&command_sighand_struct,
+#else
+	&command_signal_struct,
+#endif
 	&command_slab,
 	&command_sock,
 	&command_socket,
+#if defined(CONFIG_NFSD) || defined(CONFIG_NFSD_MODULE)
+	&command_svc_client,
+	&command_svc_export,
+#endif
 	&command_swap_info,
 	&command_tq_struct,
 	&command_timer_vec,
-#ifdef CONFIG_USB
+	&command_tty_struct,
+#if defined(CONFIG_USB) || defined(CONFIG_USB_MODULE)
 	&command_usb_bus,
 	&command_usb_device,
 	&command_usb_driver,
 #endif
-#ifdef CONFIG_BLK_DEV_LVM
+	&command_user_struct,
+#if defined(CONFIG_BLK_DEV_LVM) || defined(CONFIG_BLK_DEV_LVM_MODULE)
 	&command_vg,
 #endif
 	&command_vm_area_struct,
