@@ -26,7 +26,7 @@ trace()
 	}
 
 	eip = esp = stacktop = 0;
-	pid = getinfo_fromtask(getaddr(args[1]), &eip, &esp, &stacktop);
+	pid = getinfo_fromtask(getvalue(args[1]), &eip, &esp, &stacktop);
 	mprintf("PID = %d\n", pid);
 	memread(stacktop, sizeof(stack), stack, "stack area");
 
