@@ -4,14 +4,20 @@
 
 #include <string.h>
 #include "crash.h"
+
+#if LINUXVER == 24 /*XXX*/
+#include <asm/processor.h>	/* for THREAD_SIZE */
+#else
+#define PAGE_SIZE	4096
+#endif
+
 #include "asm.h"
+#include "stacksize.h"		/* for STACKSIZE */
 
 PRIVATE addr_t trace();
-
 struct commandtable command_trace =
 	{"trace", trace, "address", "print kernel backtrace"};
 
-#define STACKSIZE 8192
 PRIVATE char stack[STACKSIZE];
 
 PRIVATE addr_t
