@@ -74,8 +74,8 @@ extern void printxd(const unsigned char *p, addr_t addr, int size, prmode_t);
 extern void mprintf(const char *fmt, ...)
 	__attribute__ ((format (printf, 1, 2)));
 extern void mprintbit(const struct bitname *, long);
-extern void mprint_str(const unsigned char *p, int maxlen);
-extern void pmode(int);
+extern int  mprintstr(const unsigned char *p, int maxlen);
+extern void mprint_mode(int);
 extern unsigned int m_ntohl(unsigned int);
 extern unsigned short m_ntohs(unsigned short);
 extern int search_slab(addr_t addr);
@@ -83,6 +83,9 @@ extern int search_task(addr_t addr);
 extern void vers_init(void);
 extern int getmsyms_fromfile(const char *, addr_t, addr_t, addr_t);
 extern int getinfo_fromtask(addr_t addr, addr_t *ip, addr_t *sp, addr_t *stack);
+struct semaphore;
+extern void mprint_semaphore(struct semaphore *);
+extern void mprint_jif(unsigned long);
 
 extern addr_t init_task_addr;
 #endif /*__STDC__*/
