@@ -20,6 +20,7 @@ extern const commandtable_t
 	command_fs_struct,
 	command_function,
 	command_help,
+	command_jiffies,
 	command_mm_struct,
 	command_module,
 	command_print,
@@ -28,6 +29,7 @@ extern const commandtable_t
 	command_semaphore,
 	command_set,
 	command_sizeof,
+	command_stat,
 	command_super_block,
 	command_sys_call_table,
 	command_task_struct,
@@ -49,6 +51,7 @@ const commandtable_t *commandtable_base[] = {
 	&command_fs_struct,
 	&command_function,
 	&command_help,
+	&command_jiffies,
 	&command_mm_struct,
 	&command_module,
 	&command_print,
@@ -57,6 +60,7 @@ const commandtable_t *commandtable_base[] = {
 	&command_semaphore,
 	&command_set,
 	&command_sizeof,
+	&command_stat,
 	&command_super_block,
 	&command_sys_call_table,
 	&command_task_struct,
@@ -80,7 +84,7 @@ int aliastable_size = sizeof(initial_alias) / sizeof(initial_alias[0]) / 2;
 const char *initial_function[] = {
 	"i", "if [$# <= 1]; then inode | less; else inode $*; fi",
 	"f", "if [$# <= 1]; then file | less; else file $*; fi",
-#if LINUXVER==25
+#if LINUXVER >= 25
 	"traceall", "task_struct init_task; cur=$?; trace init_task; while [$cur != init_task]; do echo; task_struct $cur; next=$?; trace $cur; cur=$next; done",
 #else
 	"traceall", "task_struct init_task_union; cur=$?; trace init_task_union; while [$cur != init_task_union]; do echo; task_struct $cur; next=$?; trace $cur; cur=$next; done",
