@@ -7,7 +7,7 @@
 
 PRIVATE addr_t xfs_log_item();
 const commandtable_t command_xfs_log_item =
-	{"xfs_log_item", xfs_log_item, "[-a] address", "[XFS] print xfs_log_item_t table\n  -a  follow list"};
+	{"xfs_log_item", xfs_log_item, "[-a] [-f] address", "[XFS] print xfs_log_item_t table\n  -a  follow list"};
 extern void prhead_xfs_log_item();
 extern addr_t print_xfs_log_item();
 
